@@ -14,17 +14,19 @@ function goto_up2k() {
 
 // chrome requires https to use crypto.subtle,
 // usually it's undefined but some chromes throw on invoke
-var up2k = null;
-var sha_js = window.WebAssembly ? 'hw' : 'ac';  // ff53,c57,sa11
+var up2k = null,
+    sha_js = window.WebAssembly ? 'hw' : 'ac',  // ff53,c57,sa11
+    m = 'will use ' + sha_js + ' instead of native sha512 due to';
+
 try {
     var cf = crypto.subtle || crypto.webkitSubtle;
     cf.digest('SHA-512', new Uint8Array(1)).then(
         function (x) { console.log('sha-ok'); up2k = up2k_init(cf); },
-        function (x) { console.log('sha-ng:', x); up2k = up2k_init(false); }
+        function (x) { console.log(m, x); up2k = up2k_init(false); }
     );
 }
 catch (ex) {
-    console.log('sha-na:', ex);
+    console.log(m, ex);
     try {
         up2k = up2k_init(false);
     }
@@ -140,7 +142,7 @@ function U2pvis(act, btns) {
     this.tail = -1;
     this.wsz = 3;
 
-    this.addfile = function (entry, sz) {
+    this.addfile = function (entry, sz, draw) {
         this.tab.push({
             "hn": entry[0],
             "ht": entry[1],
@@ -154,6 +156,9 @@ function U2pvis(act, btns) {
             "bd0": 0  // upload start
         });
         this.ctr["q"]++;
+        if (!draw)
+            return;
+
         this.drawcard("q");
         if (this.act == "q") {
             this.addrow(this.tab.length - 1);
@@ -220,7 +225,7 @@ function U2pvis(act, btns) {
     this.hashed = function (fobj) {
         var fo = this.tab[fobj.n],
             nb = fo.bt * (++fo.nh / fo.cb.length),
-            p = this.perc(nb, 0, fobj.size, fobj.t1);
+            p = this.perc(nb, 0, fobj.size, fobj.t_hashing);
 
         fo.hp = '{0}%, {1}, {2} MB/s'.format(
             p[0].toFixed(2), p[1], p[2].toFixed(2)
@@ -243,7 +248,7 @@ function U2pvis(act, btns) {
         fo.cb[nchunk] = cbd;
         fo.bd += delta;
 
-        var p = this.perc(fo.bd, fo.bd0, fo.bt, fobj.t3);
+        var p = this.perc(fo.bd, fo.bd0, fo.bt, fobj.t_uploading);
         fo.hp = '{0}%, {1}, {2} MB/s'.format(
             p[0].toFixed(2), p[1], p[2].toFixed(2)
         );
@@ -254,6 +259,41 @@ function U2pvis(act, btns) {
         var obj = ebi('f{0}p'.format(fobj.n)),
             o1 = p[0] - 2, o2 = p[0] - 0.1, o3 = p[0];
 
+        if (!obj) { //} || true) {
+            var msg = [
+                "act", this.act,
+                "in", fo.in,
+                "is_act", this.is_act(fo.in),
+                "head", this.head,
+                "tail", this.tail,
+                "nfile", fobj.n,
+                "name", fobj.name,
+                "sz", fobj.size,
+                "bytesDelta", delta,
+                "bytesDone", fo.bd,
+            ],
+                m2 = '',
+                ds = QSA("#u2tab>tbody>tr>td:first-child>a:last-child");
+
+            for (var a = 0; a < msg.length; a += 2)
+                m2 += msg[a] + '=' + msg[a + 1] + ', ';
+
+            console.log(m2);
+
+            for (var a = 0, aa = ds.length; a < aa; a++) {
+                var id = ds[a].parentNode.getAttribute('id').slice(1, -1);
+                console.log("dom %d/%d = [%s] in(%s) is_act(%s) %s",
+                    a, aa, id, this.tab[id].in, this.is_act(fo.in), ds[a].textContent);
+            }
+
+            for (var a = 0, aa = this.tab.length; a < aa; a++)
+                if (this.is_act(this.tab[a].in))
+                    console.log("tab %d/%d = sz %s", a, aa, this.tab[a].bt);
+
+            console.log("a");
+            throw 42;
+        }
+
         obj.innerHTML = fo.hp;
         obj.style.color = '#fff';
         obj.style.background = 'linear-gradient(90deg, #050, #270 ' + o1 + '%, #4b0 ' + o2 + '%, #333 ' + o3 + '%, #333 99%, #777)';
@@ -268,26 +308,35 @@ function U2pvis(act, btns) {
             throw 42;
         }
 
+        //console.log("oldcat %s %d, newcat %s %d, head=%d, tail=%d, file=%d, act.old=%s, act.new=%s, bz_act=%s",
+        //    oldcat, this.ctr[oldcat],
+        //    newcat, this.ctr[newcat],
+        //    this.head, this.tail, nfile,
+        //    this.is_act(oldcat), this.is_act(newcat), bz_act);
+
         fo.in = newcat;
         this.ctr[oldcat]--;
         this.ctr[newcat]++;
         this.drawcard(oldcat);
         this.drawcard(newcat);
         if (this.is_act(newcat)) {
-            this.tail++;
+            this.tail = Math.max(this.tail, nfile + 1);
             if (!ebi('f' + nfile))
                 this.addrow(nfile);
         }
         else if (this.is_act(oldcat)) {
-            this.head++;
+            while (this.head < Math.min(this.tab.length, this.tail) && this.precard[this.tab[this.head].in])
+                this.head++;
+
             if (!bz_act) {
                 var tr = ebi("f" + nfile);
                 tr.parentNode.removeChild(tr);
             }
         }
-        if (bz_act) {
+        else return;
+
+        if (bz_act)
             this.bzw();
-        }
     };
 
     this.bzw = function () {
@@ -301,7 +350,8 @@ function U2pvis(act, btns) {
 
         while (this.head - first > this.wsz) {
             var obj = ebi('f' + (first++));
-            obj.parentNode.removeChild(obj);
+            if (obj)
+                obj.parentNode.removeChild(obj);
         }
         while (last - this.tail < this.wsz && last < this.tab.length - 2) {
             var obj = ebi('f' + (++last));
@@ -334,6 +384,8 @@ function U2pvis(act, btns) {
 
     this.changecard = function (card) {
         this.act = card;
+        this.precard = has(["ok", "ng", "done"], this.act) ? {} : this.act == "bz" ? { "ok": 1, "ng": 1 } : { "ok": 1, "ng": 1, "bz": 1 };
+        this.postcard = has(["ok", "ng", "done"], this.act) ? { "bz": 1, "q": 1 } : this.act == "bz" ? { "q": 1 } : {};
         this.head = -1;
         this.tail = -1;
         var html = [];
@@ -348,8 +400,18 @@ function U2pvis(act, btns) {
             }
         }
         if (this.head == -1) {
-            this.head = this.tab.length;
-            this.tail = this.head - 1;
+            for (var a = 0; a < this.tab.length; a++) {
+                var rt = this.tab[a].in;
+                if (this.precard[rt]) {
+                    this.head = a + 1;
+                    this.tail = a;
+                }
+                else if (this.postcard[rt]) {
+                    this.head = a;
+                    this.tail = a - 1;
+                    break;
+                }
+            }
         }
         if (card == "bz") {
             for (var a = this.head - 1; a >= this.head - this.wsz && a >= 0; a--) {
@@ -397,6 +459,8 @@ function U2pvis(act, btns) {
             that.changecard(newtab);
         };
     }
+
+    this.changecard(this.act);
 }
 
 
@@ -493,17 +557,21 @@ function up2k_init(subtle) {
         ask_up = bcfg_get('ask_up', true),
         flag_en = bcfg_get('flag_en', false),
         fsearch = bcfg_get('fsearch', false),
+        turbo = bcfg_get('u2turbo', false),
+        datechk = bcfg_get('u2tdate', true),
         fdom_ctr = 0,
         min_filebuf = 0;
 
     var st = {
         "files": [],
         "todo": {
+            "head": [],
             "hash": [],
             "handshake": [],
             "upload": []
         },
         "busy": {
+            "head": [],
             "hash": [],
             "handshake": [],
             "upload": []
@@ -514,6 +582,15 @@ function up2k_init(subtle) {
         }
     };
 
+    function push_t(arr, t) {
+        var sort = arr.length && arr[arr.length - 1].n > t.n;
+        arr.push(t);
+        if (sort)
+            arr.sort(function (a, b) {
+                return a.n < b.n ? -1 : 1;
+            });
+    }
+
     var pvis = new U2pvis("bz", '#u2cards');
 
     var bobslice = null;
@@ -557,7 +634,7 @@ function up2k_init(subtle) {
         }
         else files = e.target.files;
 
-        if (!files || files.length == 0)
+        if (!files || !files.length)
             return alert('no files selected??');
 
         more_one_file();
@@ -596,14 +673,50 @@ function up2k_init(subtle) {
         }
     }
 
-    function read_dirs(rd, pf, dirs, good, bad) {
+    function rd_flatten(pf, dirs) {
+        var ret = jcp(pf);
+        for (var a = 0; a < dirs.length; a++)
+            ret.push(dirs.fullPath || '');
+
+        ret.sort();
+        return ret;
+    }
+
+    var rd_missing_ref = [];
+    function read_dirs(rd, pf, dirs, good, bad, spins) {
+        spins = spins || 0;
+        if (++spins == 5)
+            rd_missing_ref = rd_flatten(pf, dirs);
+
+        if (spins == 200) {
+            var missing = rd_flatten(pf, dirs),
+                match = rd_missing_ref.length == missing.length,
+                aa = match ? missing.length : 0;
+
+            missing.sort();
+            for (var a = 0; a < aa; a++)
+                if (rd_missing_ref[a] != missing[a])
+                    match = false;
+
+            if (match) {
+                var msg = ['directory iterator got stuck on the following {0} items; good chance your browser is about to spinlock:'.format(missing.length)];
+                for (var a = 0; a < Math.min(20, missing.length); a++)
+                    msg.push(missing[a]);
+
+                alert(msg.join('\n-- '));
+                dirs = [];
+                pf = [];
+            }
+            spins = 0;
+        }
+
         if (!dirs.length) {
             if (!pf.length)
                 return gotallfiles(good, bad);
 
             console.log("retry pf, " + pf.length);
             setTimeout(function () {
-                read_dirs(rd, pf, dirs, good, bad);
+                read_dirs(rd, pf, dirs, good, bad, spins);
             }, 50);
             return;
         }
@@ -624,8 +737,7 @@ function up2k_init(subtle) {
 
                     pf.push(name);
                     dn.file(function (fobj) {
-                        var idx = pf.indexOf(name);
-                        pf.splice(idx, 1);
+                        apop(pf, name);
                         try {
                             if (fobj.size > 0) {
                                 good.push([fobj, name]);
@@ -643,12 +755,12 @@ function up2k_init(subtle) {
                 dirs.shift();
                 rd = null;
             }
-            return read_dirs(rd, pf, dirs, good, bad);
+            return read_dirs(rd, pf, dirs, good, bad, spins);
         });
     }
 
     function gotallfiles(good_files, bad_files) {
-        if (bad_files.length > 0) {
+        if (bad_files.length) {
             var ntot = bad_files.length + good_files.length,
                 msg = 'These {0} files (of {1} total) were skipped because they are empty:\n'.format(bad_files.length, ntot);
 
@@ -668,40 +780,52 @@ function up2k_init(subtle) {
         if (ask_up && !fsearch && !confirm(msg.join('\n')))
             return;
 
+        var seen = {},
+            evpath = get_evpath(),
+            draw_each = good_files.length < 50;
+
+        for (var a = 0; a < st.files.length; a++)
+            seen[st.files[a].name + '\n' + st.files[a].size] = 1;
+
         for (var a = 0; a < good_files.length; a++) {
             var fobj = good_files[a][0],
                 now = Date.now(),
                 lmod = fobj.lastModified || now;
 
             var entry = {
-                "n": parseInt(st.files.length.toString()),
+                "n": st.files.length,
                 "t0": now,
                 "fobj": fobj,
                 "name": good_files[a][1],
                 "size": fobj.size,
                 "lmod": lmod / 1000,
-                "purl": get_evpath(),
+                "purl": evpath,
                 "done": false,
                 "hash": []
-            };
+            },
+                key = entry.name + '\n' + entry.size;
 
-            var skip = false;
-            for (var b = 0; b < st.files.length; b++)
-                if (entry.name == st.files[b].name &&
-                    entry.size == st.files[b].size)
-                    skip = true;
-
-            if (skip)
+            if (seen[key])
                 continue;
 
+            seen[key] = 1;
+
             pvis.addfile([
                 fsearch ? esc(entry.name) : linksplit(
                     uricom_dec(entry.purl)[0] + entry.name).join(' '),
                 '📐 hash',
                 ''
-            ], fobj.size);
+            ], fobj.size, draw_each);
+
             st.files.push(entry);
-            st.todo.hash.push(entry);
+            if (turbo)
+                push_t(st.todo.head, entry);
+            else
+                push_t(st.todo.hash, entry);
+        }
+        if (!draw_each) {
+            pvis.drawcard("q");
+            pvis.changecard(pvis.act);
         }
     }
     ebi('u2btn').addEventListener('drop', gotfile, false);
@@ -737,15 +861,25 @@ function up2k_init(subtle) {
     //
 
     function handshakes_permitted() {
-        var lim = multitask ? 1 : 0;
+        if (!st.todo.handshake.length)
+            return true;
 
-        if (lim <
-            st.todo.upload.length +
-            st.busy.upload.length)
+        var cd = st.todo.handshake[0].cooldown;
+        if (cd && cd - Date.now() > 0)
             return false;
 
-        var cd = st.todo.handshake.length ? st.todo.handshake[0].cooldown : 0;
-        if (cd && cd - Date.now() > 0)
+        // keepalive or verify
+        if (st.todo.handshake[0].keepalive ||
+            st.todo.handshake[0].t_uploaded)
+            return true;
+
+        if (parallel_uploads <
+            st.busy.handshake.length)
+            return false;
+
+        if ((multitask ? parallel_uploads : 0) <
+            st.todo.upload.length +
+            st.busy.upload.length)
             return false;
 
         return true;
@@ -779,14 +913,17 @@ function up2k_init(subtle) {
 
             clearTimeout(tto);
             running = true;
-            while (true) {
-                var is_busy = 0 !=
-                    st.todo.hash.length +
-                    st.todo.handshake.length +
-                    st.todo.upload.length +
-                    st.busy.hash.length +
-                    st.busy.handshake.length +
-                    st.busy.upload.length;
+            while (window['vis_exh']) {
+                var now = Date.now(),
+                    is_busy = 0 !=
+                        st.todo.head.length +
+                        st.todo.hash.length +
+                        st.todo.handshake.length +
+                        st.todo.upload.length +
+                        st.busy.head.length +
+                        st.busy.hash.length +
+                        st.busy.handshake.length +
+                        st.busy.upload.length;
 
                 if (was_busy != is_busy) {
                     was_busy = is_busy;
@@ -797,7 +934,6 @@ function up2k_init(subtle) {
 
                 if (flag) {
                     if (is_busy) {
-                        var now = Date.now();
                         flag.take(now);
                         if (!flag.ours)
                             return defer();
@@ -809,43 +945,52 @@ function up2k_init(subtle) {
 
                 var mou_ikkai = false;
 
-                if (st.busy.handshake.length > 0 &&
-                    st.busy.handshake[0].busied < Date.now() - 30 * 1000
+                if (st.busy.handshake.length &&
+                    st.busy.handshake[0].t_busied < now - 30 * 1000
                 ) {
                     console.log("retrying stuck handshake");
                     var t = st.busy.handshake.shift();
                     st.todo.handshake.unshift(t);
                 }
 
-                if (st.todo.handshake.length > 0 &&
-                    st.busy.handshake.length == 0 && (
-                        st.todo.handshake[0].t4 || (
-                            handshakes_permitted() &&
-                            st.busy.upload.length < parallel_uploads
-                        )
-                    )
-                ) {
-                    exec_handshake();
+                var nprev = -1;
+                for (var a = 0; a < st.todo.upload.length; a++) {
+                    var nf = st.todo.upload[a].nfile;
+                    if (nprev == nf)
+                        continue;
+
+                    nprev = nf;
+                    var t = st.files[nf];
+                    if (now - t.t_busied > 1000 * 30 &&
+                        now - t.t_handshake > 1000 * (21600 - 1800)
+                    ) {
+                        apop(st.todo.handshake, t);
+                        st.todo.handshake.unshift(t);
+                        t.keepalive = true;
+                    }
+                }
+
+                if (st.todo.head.length &&
+                    st.busy.head.length < parallel_uploads) {
+                    exec_head();
                     mou_ikkai = true;
                 }
 
                 if (handshakes_permitted() &&
-                    st.todo.handshake.length > 0 &&
-                    st.busy.handshake.length == 0 &&
-                    st.busy.upload.length < parallel_uploads) {
+                    st.todo.handshake.length) {
                     exec_handshake();
                     mou_ikkai = true;
                 }
 
-                if (st.todo.upload.length > 0 &&
+                if (st.todo.upload.length &&
                     st.busy.upload.length < parallel_uploads) {
                     exec_upload();
                     mou_ikkai = true;
                 }
 
                 if (hashing_permitted() &&
-                    st.todo.hash.length > 0 &&
-                    st.busy.hash.length == 0) {
+                    st.todo.hash.length &&
+                    !st.busy.hash.length) {
                     exec_hash();
                     mou_ikkai = true;
                 }
@@ -952,7 +1097,7 @@ function up2k_init(subtle) {
 
             bpend += cdr - car;
 
-            reader.onload = function (e) {
+            function orz(e) {
                 if (!min_filebuf && nch == 1) {
                     min_filebuf = 1;
                     var td = Date.now() - t0;
@@ -962,9 +1107,30 @@ function up2k_init(subtle) {
                     }
                 }
                 hash_calc(nch, e.target.result);
+            }
+            reader.onload = function (e) {
+                try { orz(e); } catch (ex) { vis_exh(ex + '', '', '', '', ex); }
             };
             reader.onerror = function () {
-                alert('y o u   b r o k e    i t\nerror: ' + reader.error);
+                var err = reader.error + '';
+                var handled = false;
+
+                if (err.indexOf('NotReadableError') !== -1 || // win10-chrome defender
+                    err.indexOf('NotFoundError') !== -1  // macos-firefox permissions
+                ) {
+                    pvis.seth(t.n, 1, 'OS-error');
+                    pvis.seth(t.n, 2, err);
+                    handled = true;
+                }
+
+                if (handled) {
+                    pvis.move(t.n, 'ng');
+                    apop(st.busy.hash, t);
+                    st.bytes.uploaded += t.size;
+                    return tasker();
+                }
+
+                alert('y o u   b r o k e    i t\nfile: ' + t.name + '\nerror: ' + err);
             };
             reader.readAsArrayBuffer(
                 bobslice.call(t.fobj, car, cdr));
@@ -992,15 +1158,15 @@ function up2k_init(subtle) {
                     t.hash.push(hashtab[a]);
                 }
 
-                t.t2 = Date.now();
+                t.t_hashed = Date.now();
                 if (t.n == 0 && window.location.hash == '#dbg') {
-                    var spd = (t.size / ((t.t2 - t.t1) / 1000.)) / (1024 * 1024.);
-                    alert('{0} ms, {1} MB/s\n'.format(t.t2 - t.t1, spd.toFixed(3)) + t.hash.join('\n'));
+                    var spd = (t.size / ((t.t_hashed - t.t_hashing) / 1000.)) / (1024 * 1024.);
+                    alert('{0} ms, {1} MB/s\n'.format(t.t_hashed - t.t_hashing, spd.toFixed(3)) + t.hash.join('\n'));
                 }
 
                 pvis.seth(t.n, 2, 'hashing done');
                 pvis.seth(t.n, 1, '📦 wait');
-                st.busy.hash.splice(st.busy.hash.indexOf(t), 1);
+                apop(st.busy.hash, t);
                 st.todo.handshake.push(t);
                 tasker();
             };
@@ -1023,10 +1189,57 @@ function up2k_init(subtle) {
             }, 1);
         };
 
-        t.t1 = Date.now();
+        t.t_hashing = Date.now();
         segm_next();
     }
 
+    /////
+    ////
+    ///   head
+    //
+
+    function exec_head() {
+        var t = st.todo.head.shift();
+        st.busy.head.push(t);
+
+        var xhr = new XMLHttpRequest();
+        xhr.onerror = function () {
+            console.log('head onerror, retrying', t);
+            apop(st.busy.head, t);
+            st.todo.head.unshift(t);
+            tasker();
+        };
+        function orz(e) {
+            var ok = false;
+            if (xhr.status == 200) {
+                var srv_sz = xhr.getResponseHeader('Content-Length'),
+                    srv_ts = xhr.getResponseHeader('Last-Modified');
+
+                ok = t.size == srv_sz;
+                if (ok && datechk) {
+                    srv_ts = new Date(srv_ts) / 1000;
+                    ok = Math.abs(srv_ts - t.lmod) < 2;
+                }
+            }
+            apop(st.busy.head, t);
+            if (!ok)
+                return push_t(st.todo.hash, t);
+
+            t.done = true;
+            st.bytes.hashed += t.size;
+            st.bytes.uploaded += t.size;
+            pvis.seth(t.n, 1, 'YOLO');
+            pvis.seth(t.n, 2, "turbo'd");
+            pvis.move(t.n, 'ok');
+        };
+        xhr.onload = function (e) {
+            try { orz(e); } catch (ex) { vis_exh(ex + '', '', '', '', ex); }
+        };
+
+        xhr.open('HEAD', t.purl + t.name, true);
+        xhr.send();
+    }
+
     /////
     ////
     ///   handshake
@@ -1034,30 +1247,41 @@ function up2k_init(subtle) {
 
     function exec_handshake() {
         var t = st.todo.handshake.shift(),
+            keepalive = t.keepalive,
             me = Date.now();
 
         st.busy.handshake.push(t);
-        t.busied = me;
+        t.keepalive = undefined;
+        t.t_busied = me;
+
+        if (keepalive)
+            console.log("sending keepalive handshake", t);
 
         var xhr = new XMLHttpRequest();
         xhr.onerror = function () {
-            if (t.busied != me) {
+            if (t.t_busied != me) {
                 console.log('zombie handshake onerror,', t);
                 return;
             }
-            console.log('handshake onerror, retrying');
-            st.busy.handshake.splice(st.busy.handshake.indexOf(t), 1);
+            console.log('handshake onerror, retrying', t);
+            apop(st.busy.handshake, t);
             st.todo.handshake.unshift(t);
+            t.keepalive = keepalive;
             tasker();
         };
-        xhr.onload = function (e) {
-            if (t.busied != me) {
+        function orz(e) {
+            if (t.t_busied != me) {
                 console.log('zombie handshake onload,', t);
                 return;
             }
             if (xhr.status == 200) {
-                var response = JSON.parse(xhr.responseText);
+                t.t_handshake = Date.now();
+                if (keepalive) {
+                    apop(st.busy.handshake, t);
+                    return;
+                }
 
+                var response = JSON.parse(xhr.responseText);
                 if (!response.name) {
                     var msg = '',
                         smsg = '';
@@ -1081,7 +1305,7 @@ function up2k_init(subtle) {
                     pvis.seth(t.n, 2, msg);
                     pvis.seth(t.n, 1, smsg);
                     pvis.move(t.n, smsg == '404' ? 'ng' : 'ok');
-                    st.busy.handshake.splice(st.busy.handshake.indexOf(t), 1);
+                    apop(st.busy.handshake, t);
                     st.bytes.uploaded += t.size;
                     t.done = true;
                     tasker();
@@ -1090,6 +1314,7 @@ function up2k_init(subtle) {
 
                 if (response.name !== t.name) {
                     // file exists; server renamed us
+                    console.log("server-rename [" + t.name + "] to [" + response.name + "]");
                     t.name = response.name;
                     pvis.seth(t.n, 0, linksplit(t.purl + t.name).join(' '));
                 }
@@ -1122,31 +1347,41 @@ function up2k_init(subtle) {
                 var done = true,
                     msg = '&#x1f3b7;&#x1f41b;';
 
-                if (t.postlist.length > 0) {
+                if (t.postlist.length) {
+                    var arr = st.todo.upload,
+                        sort = arr.length && arr[arr.length - 1].nfile > t.n;
+
                     for (var a = 0; a < t.postlist.length; a++)
-                        st.todo.upload.push({
+                        arr.push({
                             'nfile': t.n,
                             'npart': t.postlist[a]
                         });
 
                     msg = 'uploading';
                     done = false;
+
+                    if (sort)
+                        arr.sort(function (a, b) {
+                            return a.nfile < b.nfile ? -1 :
+                            /*  */ a.nfile > b.nfile ? 1 :
+                                    a.npart < b.npart ? -1 : 1;
+                        });
                 }
                 pvis.seth(t.n, 1, msg);
-                st.busy.handshake.splice(st.busy.handshake.indexOf(t), 1);
+                apop(st.busy.handshake, t);
 
                 if (done) {
                     t.done = true;
                     st.bytes.uploaded += t.size - t.bytes_uploaded;
-                    var spd1 = (t.size / ((t.t2 - t.t1) / 1000.)) / (1024 * 1024.),
-                        spd2 = (t.size / ((t.t4 - t.t3) / 1000.)) / (1024 * 1024.);
+                    var spd1 = (t.size / ((t.t_hashed - t.t_hashing) / 1000.)) / (1024 * 1024.),
+                        spd2 = (t.size / ((t.t_uploaded - t.t_uploading) / 1000.)) / (1024 * 1024.);
 
                     pvis.seth(t.n, 2, 'hash {0}, up {1} MB/s'.format(
                         spd1.toFixed(2), spd2.toFixed(2)));
 
                     pvis.move(t.n, 'ok');
                 }
-                else t.t4 = undefined;
+                else t.t_uploaded = undefined;
 
                 tasker();
             }
@@ -1165,7 +1400,7 @@ function up2k_init(subtle) {
                     var penalty = rsp.replace(/.*rate-limit /, "").split(' ')[0];
                     console.log("rate-limit: " + penalty);
                     t.cooldown = Date.now() + parseFloat(penalty) * 1000;
-                    st.busy.handshake.splice(st.busy.handshake.indexOf(t), 1);
+                    apop(st.busy.handshake, t);
                     st.todo.handshake.unshift(t);
                     return;
                 }
@@ -1184,7 +1419,7 @@ function up2k_init(subtle) {
                     pvis.seth(t.n, 2, err);
                     pvis.move(t.n, 'ng');
 
-                    st.busy.handshake.splice(st.busy.handshake.indexOf(t), 1);
+                    apop(st.busy.handshake, t);
                     tasker();
                     return;
                 }
@@ -1194,6 +1429,9 @@ function up2k_init(subtle) {
                         (xhr.responseText && xhr.responseText) ||
                         "no further information"));
             }
+        }
+        xhr.onload = function (e) {
+            try { orz(e); } catch (ex) { vis_exh(ex + '', '', '', '', ex); }
         };
 
         var req = {
@@ -1222,8 +1460,8 @@ function up2k_init(subtle) {
         var npart = upt.npart,
             t = st.files[upt.nfile];
 
-        if (!t.t3)
-            t.t3 = Date.now();
+        if (!t.t_uploading)
+            t.t_uploading = Date.now();
 
         pvis.seth(t.n, 1, "🚀 send");
 
@@ -1234,40 +1472,56 @@ function up2k_init(subtle) {
         if (cdr >= t.size)
             cdr = t.size;
 
-        var xhr = new XMLHttpRequest();
-        xhr.upload.onprogress = function (xev) {
-            pvis.prog(t, npart, xev.loaded);
-        };
-        xhr.onload = function (xev) {
+        function orz(xhr) {
+            var txt = ((xhr.response && xhr.response.err) || xhr.responseText) + '';
             if (xhr.status == 200) {
                 pvis.prog(t, npart, cdr - car);
                 st.bytes.uploaded += cdr - car;
                 t.bytes_uploaded += cdr - car;
-                st.busy.upload.splice(st.busy.upload.indexOf(upt), 1);
-                t.postlist.splice(t.postlist.indexOf(npart), 1);
-                if (t.postlist.length == 0) {
-                    t.t4 = Date.now();
-                    pvis.seth(t.n, 1, 'verifying');
-                    st.todo.handshake.unshift(t);
-                }
-                tasker();
             }
-            else
+            else if (txt.indexOf('already got that') !== -1) {
+                console.log("ignoring dupe-segment error", t);
+            }
+            else {
                 alert("server broke; cu-err {0} on file [{1}]:\n".format(
-                    xhr.status, t.name) + (
-                        (xhr.response && xhr.response.err) ||
-                        (xhr.responseText && xhr.responseText) ||
-                        "no further information"));
-        };
-        xhr.open('POST', t.purl + 'chunkpit.php', true);
-        xhr.setRequestHeader("X-Up2k-Hash", t.hash[npart]);
-        xhr.setRequestHeader("X-Up2k-Wark", t.wark);
-        xhr.setRequestHeader('Content-Type', 'application/octet-stream');
-        if (xhr.overrideMimeType)
-            xhr.overrideMimeType('Content-Type', 'application/octet-stream');
+                    xhr.status, t.name) + (txt || "no further information"));
+                return;
+            }
+            apop(st.busy.upload, upt);
+            apop(t.postlist, npart);
+            if (!t.postlist.length) {
+                t.t_uploaded = Date.now();
+                pvis.seth(t.n, 1, 'verifying');
+                st.todo.handshake.unshift(t);
+            }
+            tasker();
+        }
+        function do_send() {
+            var xhr = new XMLHttpRequest();
+            xhr.upload.onprogress = function (xev) {
+                pvis.prog(t, npart, xev.loaded);
+            };
+            xhr.onload = function (xev) {
+                try { orz(xhr); } catch (ex) { vis_exh(ex + '', '', '', '', ex); }
+            };
+            xhr.onerror = function (xev) {
+                if (!window['vis_exh'])
+                    return;
 
-        xhr.responseType = 'text';
-        xhr.send(bobslice.call(t.fobj, car, cdr));
+                console.log('chunkpit onerror, retrying', t);
+                do_send();
+            };
+            xhr.open('POST', t.purl + 'chunkpit.php', true);
+            xhr.setRequestHeader("X-Up2k-Hash", t.hash[npart]);
+            xhr.setRequestHeader("X-Up2k-Wark", t.wark);
+            xhr.setRequestHeader('Content-Type', 'application/octet-stream');
+            if (xhr.overrideMimeType)
+                xhr.overrideMimeType('Content-Type', 'application/octet-stream');
+
+            xhr.responseType = 'text';
+            xhr.send(bobslice.call(t.fobj, car, cdr));
+        }
+        do_send();
     }
 
     /////
@@ -1363,6 +1617,30 @@ function up2k_init(subtle) {
         set_fsearch(!fsearch);
     }
 
+    function tgl_turbo() {
+        turbo = !turbo;
+        bcfg_set('u2turbo', turbo);
+        draw_turbo();
+    }
+
+    function tgl_datechk() {
+        datechk = !datechk;
+        bcfg_set('u2tdate', datechk);
+    }
+
+    function draw_turbo() {
+        var msg = '<p class="warn">WARNING: turbo enabled, <span>&nbsp;client may not detect and resume incomplete uploads; see turbo-button tooltip</span></p>',
+            html = ebi('u2foot').innerHTML;
+
+        if (turbo && html.indexOf(msg) === -1)
+            html += msg;
+        else if (!turbo)
+            html = html.replace(msg, '');
+
+        ebi('u2foot').innerHTML = html;
+    }
+    draw_turbo();
+
     function set_fsearch(new_state) {
         var fixed = false;
 
@@ -1444,6 +1722,8 @@ function up2k_init(subtle) {
     ebi('multitask').addEventListener('click', tgl_multitask, false);
     ebi('ask_up').addEventListener('click', tgl_ask_up, false);
     ebi('flag_en').addEventListener('click', tgl_flag_en, false);
+    ebi('u2turbo').addEventListener('click', tgl_turbo, false);
+    ebi('u2tdate').addEventListener('click', tgl_datechk, false);
     var o = ebi('fsearch');
     if (o)
         o.addEventListener('click', tgl_fsearch, false);
